from typing import Any, Dict
from chroma_feedback import helper


def normalize_data(build : Dict[str, Any]) -> Dict[str, Any]:
	return\
	{
		'producer': 'github',
		'slug': build['repository']['full_name'],
		'active': True,
		'status': normalize_status(build['status'], build['conclusion'])
	}


def normalize_status(status : str, conclusion : str) -> str:
	status = helper.to_lower_case(status)

	if status in ['in_progress', 'queued']:
		return 'started'
	if conclusion in ['cancelled', 'timed_out']:
		return 'errored'
	if conclusion == 'failure':
		return 'failed'
	return 'passed'



from typing import Any, Dict, List
from argparse import ArgumentParser
from chroma_feedback import helper, request
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--appveyor-host', default = 'https://ci.appveyor.com')
		program.add_argument('--appveyor-slug', action = 'append')
		program.add_argument('--appveyor-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []

	if ARGS.appveyor_slug:
		for slug in ARGS.appveyor_slug:
			result.extend(fetch(ARGS.appveyor_host, slug, None))
	else:
		result.extend(fetch(ARGS.appveyor_host, None, ARGS.appveyor_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/api/projects/' + slug, headers =
		{
			'Accept': 'application/json',
			'Authorization': 'Bearer ' + token
		})
	elif host and token:
		response = request.get(host + '/api/projects', headers =
		{
			'Accept': 'application/json',
			'Authorization': 'Bearer ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'project' and 'build' in data:
			result.append(normalize_data(data['project'], data['build']))
		if 'builds' in helper.get_first(data):
			for project in data:
				build = helper.get_first(project['builds'])
				if build:
					result.append(normalize_data(project, build))
	return result



from typing import Any, Dict, List
from chroma_feedback import color
from .api import get_api, get_loop, get_builder


def get_lights(ips : List[str]) -> List[Dict[str, Any]]:
	lights = []

	for ip in ips:
		lights.append(get_api(ip))
	return lights


def process_lights(lights : Any, status : str) -> List[Dict[str, Any]]:
	result = []

	# process lights

	for light in lights:
		light_name = get_light_name(light)

		if status == 'passed':
			result.append(
			{
				'consumer': 'wiz_light',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_passed()),
				'status': status
			})
		if status == 'started':
			result.append(
			{
				'consumer': 'wiz_light',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_started()),
				'status': status
			})
		if status == 'errored':
			result.append(
			{
				'consumer': 'wiz_light',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_errored()),
				'status': status
			})
		if status == 'failed':
			result.append(
			{
				'consumer': 'wiz_light',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_failed()),
				'status': status
			})
	return result


def get_light_name(light : Any) -> str:
	loop = get_loop()
	config = loop.run_until_complete(light.get_bulbtype())
	return config.name


def static_light(light : Any, color_config : Dict[str, Any]) -> bool:
	loop = get_loop()
	builder = get_builder()
	loop.run_until_complete(light.turn_on(builder(rgb = color_config['rgb'])))
	return loop.is_closed() is False



from typing import Any, Dict, List
from argparse import ArgumentParser
from chroma_feedback import helper, request
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--circle-host', default = 'https://circleci.com')
		program.add_argument('--circle-slug', action = 'append')
		program.add_argument('--circle-organization')
		program.add_argument('--circle-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []

	if ARGS.circle_slug:
		for slug in ARGS.circle_slug:
			result.extend(fetch(ARGS.circle_host, None, slug, ARGS.circle_token))
	elif ARGS.circle_organization:
		result.extend(fetch(ARGS.circle_host, ARGS.circle_organization, None, ARGS.circle_token))
	return result


def fetch(host : str, organization : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/api/v2/project/' + slug + '/pipeline', headers =
		{
			'Accept': 'application/json',
			'Circle-Token': token
		})
	elif host and organization and token:
		response = request.get(host + '/api/v2/pipeline?org-slug=' + organization, headers =
		{
			'Accept': 'application/json',
			'Circle-Token': token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'items' in data:
			pipeline = helper.get_first(data['items'])
			if 'id' in pipeline:
				result.extend(fetch_workflows(host, pipeline['id'], token))
	return result


def fetch_workflows(host : str, pipeline_id : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and pipeline_id and token:
		response = request.get(host + '/api/v2/pipeline/' + pipeline_id + '/workflow', headers =
		{
			'Accept': 'application/json',
			'Circle-Token': token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'items' in data:
			for build in data['items']:
				result.append(normalize_data(build))
	return result



from typing import Any, Dict, List
import copy
from chroma_feedback import color


def get_devices(devices : Any, device_names : List[str]) -> Any:
	if device_names:
		for device in copy.copy(devices):
			if device.name not in device_names:
				devices.remove(device)
	return devices


def process_devices(devices : Any, status : str) -> List[Dict[str, Any]]:
	result = []

	# process devices

	for device in devices:
		if status == 'passed':
			result.append(
			{
				'consumer': 'razer_chroma',
				'type': 'device',
				'name': device.name,
				'active': static_device(device, color.get_passed()),
				'status': status
			})
		if status == 'started':
			result.append(
			{
				'consumer': 'razer_chroma',
				'type': 'device',
				'name': device.name,
				'active': static_device(device, color.get_started()),
				'status': status
			})
		if status == 'errored':
			result.append(
			{
				'consumer': 'razer_chroma',
				'type': 'device',
				'name': device.name,
				'active': pulsate_device(device, color.get_errored()) or breath_device(device, color.get_errored()),
				'status': status
			})
		if status == 'failed':
			result.append(
			{
				'consumer': 'razer_chroma',
				'type': 'device',
				'name': device.name,
				'active': pulsate_device(device, color.get_failed()) or breath_device(device, color.get_failed()),
				'status': status
			})
	return result


def static_device(device : Any, color_config : Dict[str, Any]) -> bool:
	if device.has('brightness'):
		device.brightness = color_config['brightness'][0]
	if device.fx.has('static'):
		return device.fx.static(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2])
	return misc_effect(device, color_config, 'static')


def pulsate_device(device : Any, color_config : Dict[str, Any]) -> bool:
	if device.has('brightness'):
		device.brightness = color_config['brightness'][0]
	if device.fx.has('pulsate'):
		return device.fx.breath_single(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2])
	return misc_effect(device, color_config, 'pulsate')


def breath_device(device : Any, color_config : Dict[str, Any]) -> bool:
	if device.has('brightness'):
		device.brightness = color_config['brightness'][0]
	if device.fx.has('breath_single'):
		return device.fx.breath_single(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2])
	return misc_effect(device, color_config, 'breath_single')


def misc_effect(device : Any, color_config : Dict[str, Any], effect_name : str) -> bool:
	effect_state = False
	parts =\
	{
		'logo': 'logo',
		'scroll': 'scroll_wheel',
		'left': 'left',
		'right': 'right',
		'backlight': 'backlight'
	}

	for part_key, part_value in parts.items():
		if device.fx.has(part_key + '_' + effect_name):
			effect_function = getattr(getattr(device.fx.misc, part_value), effect_name)
			effect_state = effect_function(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2]) or effect_state
	return effect_state



from typing import Any, Dict, List
from argparse import ArgumentParser
from chroma_feedback import helper, request
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--github-host', default = 'https://api.github.com')
		program.add_argument('--github-slug', action = 'append', required = True)
		program.add_argument('--github-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []

	for slug in ARGS.github_slug:
		result.extend(fetch(ARGS.github_host, slug, ARGS.github_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	repositories = fetch_repositories(host, slug, token)

	if repositories:
		for repository in repositories:
			if 'full_name' in repository:
				result.extend(fetch_runs(host, repository['full_name'], token))
	else:
		result.extend(fetch_runs(host, slug, token))
	return result


def fetch_repositories(host : str, username : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and username and token:
		response = request.get(host + '/users/' + username + '/repos', headers =
		{
			'Accept': 'application/vnd.github.v3+json',
			'Authorization': 'Token ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if data:
			for project in data:
				result.append(project)
	return result


def fetch_runs(host : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/repos/' + slug + '/actions/runs', headers =
		{
			'Accept': 'application/vnd.github.v3+json',
			'Authorization': 'Token ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'workflow_runs' in data:
			build = helper.get_first(data['workflow_runs'])
			if build:
				result.append(normalize_data(build))
	return result



import asyncio
from typing import Any
from chroma_feedback import wording

API = None
LOOP = None


def get_api(ip : str) -> Any:
	global API

	if not API:
		API = api_factory(ip)
	return API


def api_factory(ip : str) -> Any:
	api = None

	try:
		from pywizlight import wizlight
		from pywizlight.exceptions import WizLightError

		try:
			api = wizlight(ip)
			get_loop().run_until_complete(api.updateState())
		except WizLightError:
			exit(wording.get('connection_no').format('WIZ LIGHT') + wording.get('exclamation_mark'))
		return api
	except ImportError:
		exit(wording.get('package_no').format('WIZ LIGHT') + wording.get('exclamation_mark'))


def get_loop() -> Any:
	global LOOP

	if not LOOP:
		LOOP = asyncio.get_event_loop()
	return LOOP


def get_builder() -> Any:
	try:
		from pywizlight import PilotBuilder

		return PilotBuilder
	except ImportError:
		return None



from typing import Any, List
import tempfile
import os
from PIL import Image, ImageDraw
from chroma_feedback import color, metadata, wording
try:
	import gi

	gi.require_version('Gtk', '3.0')
	gi.require_version('AppIndicator3', '0.1')

	from gi.repository import AppIndicator3, GdkPixbuf, Gtk
except (ImportError, ValueError):
	exit(wording.get('package_no').format('GIRL APPINDICATOR') + wording.get('exclamation_mark'))

SYSTRAY = None


def create(status : str, report : List[str]) -> None:
	global SYSTRAY

	if not SYSTRAY:
		icon_path = create_icon(status)
		SYSTRAY = AppIndicator3.Indicator.new(metadata.get('name') + ' ' + metadata.get('version'), icon_path, AppIndicator3.IndicatorCategory.APPLICATION_STATUS)
		SYSTRAY.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
		SYSTRAY.set_menu(create_menu(report))
	Gtk.main()


def update(status : str, report : List[str]) -> None:
	icon_path = create_icon(status)

	SYSTRAY.set_icon(icon_path)
	SYSTRAY.set_menu(create_menu(report))


def is_active() -> bool:
	global SYSTRAY

	return SYSTRAY is not None


def create_menu(report : List[str]) ->Gtk.Menu:
	menu = Gtk.Menu()

	# process report

	for message in report:
		menu.append(Gtk.MenuItem(message))
	if report:
		menu.append(Gtk.SeparatorMenuItem())

	# handle action

	item_about = Gtk.MenuItem(wording.get('about'))
	item_about.connect('activate', about)
	item_exit = Gtk.MenuItem(wording.get('exit'))
	item_exit.connect('activate', destroy)
	menu.append(item_about)
	menu.append(item_exit)
	menu.show_all()
	return menu


def create_icon(status : str) -> str:
	color_config = color.get_by_status(status)
	image = Image.new('RGBA', (100, 100), (0, 0, 0, 0))
	draw = ImageDraw.Draw(image)
	draw.ellipse((20, 20, 80, 80), fill = (color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2]))
	path = tempfile.mktemp('.png')
	image.save(path)
	return path


def about(menu_item : Any) -> None:
	logo = GdkPixbuf.Pixbuf.new_from_file_at_size(create_icon('passed'), 20, 20)
	about_dialog = Gtk.AboutDialog()
	about_dialog.set_program_name(metadata.get('name'))
	about_dialog.set_comments(metadata.get('description'))
	about_dialog.set_version(metadata.get('version'))
	about_dialog.set_website(metadata.get('url'))
	about_dialog.set_website_label(metadata.get('url'))
	about_dialog.set_logo(logo)
	about_dialog.present()


def destroy(menu_item : Any) -> None:
	print('\r' + wording.get('goodbye') + wording.get('exclamation_mark'))
	os._exit(0)



#!/usr/bin/env python3

from setuptools import setup
from chroma_feedback import metadata

setup(
	name = metadata.get('name'),
	description = metadata.get('description'),
	long_description = open('README.md').read(),
	long_description_content_type = 'text/markdown',
	version = metadata.get('version'),
	license = metadata.get('license'),
	keywords = metadata.get('keywords'),
	author = metadata.get('author'),
	author_email = metadata.get('author_email'),
	url = metadata.get('url'),
	packages =
	[
		'chroma_feedback',
		'chroma_feedback.producer',
		'chroma_feedback.producer.appveyor',
		'chroma_feedback.producer.bamboo',
		'chroma_feedback.producer.bitbucket',
		'chroma_feedback.producer.circle',
		'chroma_feedback.producer.codeship',
		'chroma_feedback.producer.github',
		'chroma_feedback.producer.gitlab',
		'chroma_feedback.producer.jenkins',
		'chroma_feedback.producer.teamcity',
		'chroma_feedback.producer.travis',
		'chroma_feedback.producer.wercker',
		'chroma_feedback.consumer',
		'chroma_feedback.consumer.agile_innovative_blinkstick',
		'chroma_feedback.consumer.lifx_light',
		'chroma_feedback.consumer.magic_hue',
		'chroma_feedback.consumer.nanoleaf_light',
		'chroma_feedback.consumer.philips_hue',
		'chroma_feedback.consumer.razer_chroma',
		'chroma_feedback.consumer.thingm_blink',
		'chroma_feedback.consumer.wiz_light',
		'chroma_feedback.consumer.xiaomi_yeelight'
	],
	scripts =
	[
		'bin/chroma-feedback'
	],
	install_requires =
	[
		'asyncio==3.4.3',
		'blink1==0.3.1',
		'blinkstick==1.1.8',
		'lifxlan==1.2.5',
		'nanoleafapi==2.0.0',
		'phue==1.1',
		'pillow==8.1.2',
		'python-magichue==0.2.8',
		'pywizlight==0.4.2',
		'requests==2.25.1',
		'yeelight==0.5.1'
	],
	tests_require =
	[
		'coveralls==2.9.3',
		'pylint==2.6.0',
		'pytest==6.2.1',
		'pytest-cov==2.10.1',
		'pytest-mock==3.5.0',
		'mock==4.0.3',
		'mypy==0.790'
	]
)



from typing import Any
from chroma_feedback import wording

API = None


def get_api(ip : str) -> Any:
	global API

	if not API:
		API = api_factory(ip)
	return API


def api_factory(ip : str) -> Any:
	api = None

	try:
		from nanoleafapi import Nanoleaf, NanoleafConnectionError, NanoleafRegistrationError

		try:
			api = Nanoleaf(ip)
		except NanoleafConnectionError:
			exit(wording.get('connection_no').format('NANOLEAF LIGHT') + wording.get('exclamation_mark'))
		except NanoleafRegistrationError:
			exit(wording.get('press_button').format('PAIRING', 'NANOLEAF LIGHT') + wording.get('exclamation_mark'))
		return api
	except ImportError:
		exit(wording.get('package_no').format('NANOLEAF LIGHT') + wording.get('exclamation_mark'))
